#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include "main.h"




const unsigned char char_table1[] =
{
    ' ','1','2','3','4','5','6','7','8','9','0',
    'A','B','C','D','E','F','G','H','I','J','K',
    'L','M','N','O','P','Q','R','S','T','U','V',
    'W','X','Y','Z','a','b','c','d','e','f','g',
    'h','i','j','k','l','m','n','o','p','q','r',
    's','t','u','v','w','x','y','z','(',')','.',
    '?',';',':',
    ',',
    0x27,
    '/',
    '"',
    '-',
    '<',
    '>',
    '@',
};


const unsigned char char_to_char_code_table1[] =
{
    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,
    0x0B,0x0C,0x0D,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,
    0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x20,
    0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,
    0x2C,0x2D,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x36,
    0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,
    0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,
};


const unsigned char AKKU100[] =
{0xED,0xEE,0xEE,0xEE,0xEE,0xF3,

};
const unsigned char AKKU5[] =
{0xED,0xF0,0xF2,0xF2,0xF2,0xF3,

};



unsigned char get_the_char_code_from_table (unsigned char test_char)
{
    unsigned char count;
    unsigned char rv_char;

    rv_char = 0x00;

    for (count = 0; count < sizeof(char_table1); count++)
    {
        if (test_char == char_table1[count])
        {
            rv_char = char_to_char_code_table1[count];
            //put_char(rv_char);
            break;
        }
    }

    if (count >= sizeof(char_table1))
        rv_char = 0x00;

    return rv_char;
}


//SPI EINSCHALTEN
void osd_on(void)
{
osd_cs_high();
SPCR = (1<<SPE)|(1<<MSTR); //Enable SPI, SPI in Master Mode	
}

//SPI
void osd_off(void)
{
osd_cs_high();
SPCR = 0x00;    // SPI wieder disablen
}

void spiWriteReg(const unsigned char regAddr, const unsigned char regData)
{
osd_on();
osd_cs_low();
Write_SPI(regAddr);
Write_SPI(regData);
osd_cs_high();
osd_off();
}

void spiWriteRegInc( const unsigned char regData)
{
osd_on();
osd_cs_low();
Write_SPI(regData);
osd_cs_high();
osd_off();
}


//OFFSET Horizontal
void osd_offset_hor(const unsigned char offset)
{
spiWriteReg(0x02,offset);
}


//OFFSET VERTIKAL
void osd_offset_vert(const unsigned char offset)
{
spiWriteReg(0x03,offset);
}

//MEMORY HIGH ADRESS
void osd_mem_high(void)
{
spiWriteReg(0x05,0xff);


}


//MEMORY LOW ADRESS
void osd_mem_low(void)
{

spiWriteReg(0x05,0x00);

}

//OSD CLEAR
void clear_osd(void)
{
spiWriteReg(VIDEO_MODE_0_WRITE,0x42);
}

//SEND CHAR
void osd_draw_char(unsigned char position,unsigned char zeichen)
{
spiWriteReg(0x06,0x32);
spiWriteReg(0x07,0x02);

}

void osd_rest_chip(void)
{
spiWriteReg(0x00,0x02);
}


//Schreibe Text auf OSD

void OSD_write_to_screen(unsigned char *data, unsigned char x, unsigned char y,unsigned short int blink,unsigned short int invert)
{
  unsigned int linepos;
   uchar char_address_hi, char_address_lo;
 unsigned char charcode = 0;

 

  char_address_hi = 0;
  char_address_lo = 0;

  // convert x,y to line position
  linepos = y*30+x;

  // divide in to hi & lo byte
  char_address_hi = linepos >> 8;
  char_address_lo = linepos;

 
  
  
  

 spiWriteReg(0x05,char_address_hi); // set start address high
 spiWriteReg(0x06,char_address_lo); // set start address low
if(blink==1 && invert==1) spiWriteReg(0x04,0x19);//BLINK /INVERT
if(blink==0 && invert==0) spiWriteReg(0x04,0x1);//NORMAL
if(blink==1 && invert==0) spiWriteReg(0x04,0x11);//BLINK
if(blink==0 && invert==1) spiWriteReg(0x04,0x9);//INVERT


while(*data)	
 {
  
  charcode =get_the_char_code_from_table(*data);  
 waitms(1);
  spiWriteRegInc(charcode);  //dmdi is character ady byte
  //WriteSpi(DMDI_REG_WRITE);  
  data++; 
  waitms(1);
  //put_char(charcode);		 	
 }
spiWriteRegInc(0xFF); //"Escape Character" to end Auto-Increment mode
spiWriteReg(0x04,0x00);


//spiWriteReg(0x07,0x0f);
  
    

   

 
  

  


}


//AKKU SYMBOL
void OSD_AKKU(unsigned short int akku_p, unsigned char x, unsigned char y,unsigned short int blink,unsigned short int invert)
{

  unsigned char count;
  unsigned int linepos;
   uchar char_address_hi, char_address_lo;

 

  char_address_hi = 0;
  char_address_lo = 0;

  // convert x,y to line position
  linepos = y*30+x;

  // divide in to hi & lo byte
  char_address_hi = linepos >> 8;
  char_address_lo = linepos;

 
  
  
  

 spiWriteReg(0x05,char_address_hi); // set start address high
 spiWriteReg(0x06,char_address_lo); // set start address low
if(blink==1 && invert==1) spiWriteReg(0x04,0x19);//BLINK /INVERT
if(blink==0 && invert==0) spiWriteReg(0x04,0x1);//NORMAL
if(blink==1 && invert==0) spiWriteReg(0x04,0x11);//BLINK
if(blink==0 && invert==1) spiWriteReg(0x04,0x9);//INVERT

switch(akku_p)
{

case 100:

    for (count = 0; count < 6; count++)
    {
            spiWriteRegInc(AKKU100[count]);
            waitms(1);
            
        
    }  
 
 break;
 
case 5:

    for (count = 0; count < 6; count++)
    {
            spiWriteRegInc(AKKU5[count]);
            waitms(1);
            
        
    }  
 
 break;
 }
 
spiWriteRegInc(0xFF); //"Escape Character" to end Auto-Increment mode
spiWriteReg(0x04,0x00);


//spiWriteReg(0x07,0x0f);
  
    

   

 
  

  


}








unsigned short int read_status_osd(void)
{
unsigned short int status;
osd_on();
osd_cs_low();
Write_SPI(0xa0);
status=Write_SPI(0xff);
osd_cs_high();
osd_off();
return status;
}